/*******************************************************************************
 * @file
 * @brief Main application
 * @author Microtronics Engineering GmbH
 * @version 01v000
 *******************************************************************************
 * @section License
 * <b>(C) Copyright 2013-2014 Microtronics Engineering GmbH,
 * http://www.microtronics.at</b>
 ******************************************************************************/
#pragma GCC optimize ("O0")

#include <zephyr.h>
#include <device.h>
#include <drivers/uart.h>
#include <sys/printk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef uint8_t u8;
typedef uint32_t u32;

#define MAIN_UART_BUF_LEN (16)


#define MAIN_DUMMYBUF_LEN (32)
u8 au8Main_DummyBuf[MAIN_DUMMYBUF_LEN] = {0};
u8 au8Main_DummyBufOld[MAIN_DUMMYBUF_LEN] = {0};
struct uart_event sMain_UartRxEv;

static void uart_callback(const struct device *dev,
			  struct uart_event *evt,
			  void *user_data)
{
	struct device *uart = user_data;
	switch (evt->type) 
  {
    case UART_TX_DONE:
    {
      printk("Tx sent %d bytes\r\n", evt->data.tx.len);
      break;
    }
    case UART_TX_ABORTED:
    {
      printk("Tx aborted\r\n");
      break;
    }
    case UART_RX_RDY:
    {
      // abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ <- just to have some text to copy
      // 12345678901234567890123456789012 <- just to have some text to copy
      int iErr;
      iErr = uart_err_check(dev);
      if(iErr)
        printk("ERR: %d\r\n", iErr);

      printk("evt len = %d, evt offs = %d\r\n", evt->data.rx.len, evt->data.rx.offset);
      printk("\"%.*s\"\r\n\n", evt->data.rx.len, &(evt->data.rx.buf[evt->data.rx.offset]));

      memcpy(&sMain_UartRxEv, evt, sizeof(sMain_UartRxEv));          /* copy just for the watch window */
      memset(au8Main_DummyBuf, 0, MAIN_DUMMYBUF_LEN);                /* just for the watch window */
      memcpy(au8Main_DummyBuf, &(evt->data.rx.buf[evt->data.rx.offset]), evt->data.rx.len); /* copy just for the watch window */

      break;
    }
    case UART_RX_BUF_REQUEST:
    {
      u8* pu8UartBuf;

      printk("buf req\r\n");
      pu8UartBuf = k_malloc(MAIN_UART_BUF_LEN);
      if(!pu8UartBuf)
      {
        printk("FAIL\r\n");
        break;
      }
      // uart_rx_buf_rsp(uart, pu8UartBuf, MAIN_UART_BUF_LEN);
      uart_rx_buf_rsp(dev, pu8UartBuf, MAIN_UART_BUF_LEN);
      break;
    }
    case UART_RX_BUF_RELEASED:
    {
      printk("free buf\r\n");
      k_free(evt->data.rx_buf.buf);
      break;
    }
    case UART_RX_DISABLED:
    {
      printk("disabled\r\n");
      break;
    }
    case UART_RX_STOPPED:
    {
      printk("stopped\r\n");
      break;
    }
  }
}

int main(void)
{
  const struct device *uart_dev  = DEVICE_DT_GET(DT_NODELABEL(uart0)); /* "printk" uart */
  const struct device *uartTest_dev = DEVICE_DT_GET(DT_NODELABEL(uart1)); /* "rx" uart */
  struct uart_config uart_cfg;
  struct uart_config uart1_cfg;
  u8* pu8UartBuf;

  uart_config_get(uart_dev, &uart_cfg);
  uart_cfg.baudrate = 115200;
  uart_configure(uart_dev, &uart_cfg); 

  uart_config_get(uartTest_dev, &uart1_cfg);
  uart1_cfg.baudrate = 115200;
  uart_configure(uartTest_dev, &uart1_cfg);  
  printk("MAIN START!\r\n");

  uart_callback_set(uartTest_dev, uart_callback, (void*)uartTest_dev);
  pu8UartBuf = k_malloc(MAIN_UART_BUF_LEN);
  uart_rx_enable(uartTest_dev, pu8UartBuf, MAIN_UART_BUF_LEN, 10);
  
//  uart_callback_set(uart_dev, uart_callback, (void*)uart_dev);
//  pu8UartBuf0 = k_malloc(MAIN_UART_BUF_LEN);
//  uart_rx_enable(uart_dev, pu8UartBuf0, MAIN_UART_BUF_LEN, 10);

  memset(au8Main_DummyBuf, 0, MAIN_DUMMYBUF_LEN);
  k_sleep(K_FOREVER);
  while(1)
  {
  }
}
